// Copyright (c) 2024 Celestino Amoroso (celestino.amoroso@gmail.com).
// All rights reserved.

// operator-insert.go
package expr

//-------- insert term

func newInsertTerm(tk *Token) (inst *term) {
	return &term{
		tk:       *tk,
		children: make([]*term, 0, 2),
		position: posInfix,
		priority: priAssign,
		evalFunc: evalInsert,
	}
}

func newAppendTerm(tk *Token) (inst *term) {
	return &term{
		tk:       *tk,
		children: make([]*term, 0, 2),
		position: posInfix,
		priority: priAssign,
		evalFunc: evalAppend,
	}
}

func evalInsert(ctx ExprContext, self *term) (v any, err error) {
	var leftValue, rightValue any

	if leftValue, rightValue, err = self.evalInfix(ctx); err != nil {
		return
	}

	if isList(rightValue) {
		list, _ := rightValue.([]any)
		v = append([]any{leftValue}, list...)
	} else {
		err = self.errIncompatibleTypes(leftValue, rightValue)
	}
	return
}

func evalAppend(ctx ExprContext, self *term) (v any, err error) {
	var leftValue, rightValue any

	if leftValue, rightValue, err = self.evalInfix(ctx); err != nil {
		return
	}

	if isList(leftValue) {
		list, _ := leftValue.([]any)
		v = append(list, rightValue)
	} else {
		err = self.errIncompatibleTypes(leftValue, rightValue)
	}
	return
}

// init
func init() {
	registerTermConstructor(SymInsert, newInsertTerm)
	registerTermConstructor(SymAppend, newAppendTerm)
}
