// Copyright (c) 2024 Celestino Amoroso (celestino.amoroso@gmail.com).
// All rights reserved.

// operator-length.go
package expr

//-------- length term

func newLengthTerm(tk *Token) (inst *term) {
	return &term{
		tk:       *tk,
		children: make([]*term, 0, 1),
		position: posPrefix,
		priority: priSign,
		evalFunc: evalLength,
	}
}

func evalLength(ctx ExprContext, self *term) (v any, err error) {
	var rightValue any

	if rightValue, err = self.evalPrefix(ctx); err != nil {
		return
	}

	if isList(rightValue) {
		list, _ := rightValue.([]any)
		v = int64(len(list))
	} else if isString(rightValue) {
		s, _ := rightValue.(string)
		v = int64(len(s))
	} else if it, ok := rightValue.(Iterator); ok {
		v = int64(it.Index())
	} else {
		err = self.errIncompatibleType(rightValue)
	}
	return
}

// init
func init() {
	registerTermConstructor(SymHash, newLengthTerm)
}
